#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Link interface.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""


class LinkConfig:
    def __init__(self) -> None:
        self.source = ""
        self.destination = ""
